@@ -9,6 +9,7 @@ use strict;
 use Carp;
 use Module::Load;
 use Module::Load::Conditional qw(can_load);
+use List::MoreUtils qw(uniq);
 
 our @extra_roles;
 
@@ -55,6 +56,35 @@ sub _merge_hash {
 	}
 }
 
+sub explist {
+	return (
+		map {
+			ref($_) eq 'ARRAY' ? @$_
+			: defined($_)      ? ( $_ )
+			:                    ( )
+		} @_
+	);
+}
+
+# get the env_keys, in original order, but use *LAST* instance
+# that retains the order specified in either default or args
+# but lets the relative order in args take precedence for keys
+# that are in both
+#
+# So, the order is:
+# [ keys that are only in default in the order they were specified in default ]
+# [ then keys that are in args in the order they were specified in args ]
+# No complaint is made if the same key is specified twice in either default
+# or args, the earlier one(s) are simply ignored.
+sub keylist {
+	my @keys;
+	for my $arg (@_) {
+		my $keys = (delete $arg->{env_keys}) // [];
+		push @keys, @$keys;
+	}
+	return (reverse uniq reverse @keys);
+}
+
 sub group {
 	my $pkg = shift;
 	my $args =
@@ -64,6 +94,20 @@ sub group {
 	# copy the default attributes as a start
 	my $use_args = {};
 	_merge_hash( $use_args, $default_attrs );
+
+	# pull out the env_keys (if any)
+	my @keys = keylist( $use_args, $args );
+	my @key_specific = map { $_->{env_key_specific} // () } $use_args, $args;
+
+	# merge any specified env_key list that has a value available
+	for my $key (@keys) {
+		for my $key_spec (@key_specific) {
+			if (my $spec_args = $key_spec->{$key}) {
+				_merge_hash( $use_args, $spec_args );
+			}
+		}
+	}
+
 	my $cluster = $args->{cluster} // $use_args->{cluster}
 		// croak("HPCI->group() requires a cluster key in the argument hash");
 
@@ -91,11 +135,11 @@ sub _trigger_mkdir {
 
 =head1 VERSION
 
-Version 0.46
+Version 0.47
 
 =cut
 
-our $VERSION = '0.46';
+our $VERSION = '0.47';
 
 our $LocalConfigFound;
 
